% The Tweety problem (L_mm LDS)
% from Mikael Asker's thesis 2003 (Lund)
% with Julien Delnaye's code as reference.

:- [altp].
:- dynamic belief/2.
:- op(950, xfy, and).

% Memory-model LDS
%
% Belief structure:
% belief(Label, Formula, RuleTag)
%
% Label is a list [loc, trigger, cert, time, pos, time-rtm]
% where
%   loc in {ltm, qtm, stm, itm};
%   trigger is a formula (0 for empty);
%   cert in {1,0};
%   time, pos and time-rtm are natural numbers.

time_step(belief([_,_,_,I,_,_],_,_), I).

% The Tweety problem
axiom([ltm,bird(X),0,0,0,0],
  bird(X) and now(I) and not(k(I-1, not(flies(X)))) -> flies(X)).
axiom([ltm,ostrich(X),1,0,0,0], ostrich(X) -> not(flies(X))).
obs(0, bird(tweety)).
obs(4, ostrich(tweety)).

rule([], belief(Label, Form, axiom)) :- axiom(Label, Form).

% Clock (A1')
rule([], belief([stm,0,1,I,I,20], now(I), clock)).

% Obs (A2')
rule([obs(I,Form)], belief([qtm,0,1,I,0,0], Form, obs)).

% Inheritance in LTM (IL)
rule([belief([ltm,Alpha,C,I,  0,0], Beta, _)],
      belief([ltm,Alpha,C,I+1,0,0], Beta, il)).

% Inheritance QTM -> STM (IQS)
rule([belief([qtm,0,C,I,0,0], Alpha, _),
      \+formula_stm(I, Alpha),
      \+formula_rtm(I, loses(Alpha))],
     belief([stm,0,C,I+1,I+1,20], Alpha, iqs)).

% Inheritance in STM (IS)
rule([belief([stm,0,C,I,P,R], Alpha, _),
      min_stm_pos(I,Pos), (P > Pos ; \+formula_new_stm(I+1, _)),
      \+formula_stm(I, contra(I-1, Alpha, _)),
      \+formula_stm(I, contra(I-1, _, Alpha)),
      \+formula_stm(I, loses(Alpha)),
      Alpha \= now(I),
      (Alpha=not(k(I-1, Beta)) -> \+formula_qtm(I, not(k(I, Beta))) ; true),
      (Alpha=contra(I-1, Beta, Gamma)
          -> \+formula_qtm(I, contra(I, Beta, Gamma)) ; true)],
     belief([stm,0,C,I+1,P,R], Alpha, is)).
% hope this works...
% NOTE: f_min_stm_pos is missing in Julien's code!!
% The two premises that are in the report:
%   [spec(f_min_stm_pos),var(i),20,var(pos)],
%   [spec(<),[-,var(pos),1],var(p)],
% are not there in the code.

% Inheritance STM -> ITM (ISI)
% almost the same
rule([belief([stm,0,C,I,P,R], Alpha, _),
      min_stm_pos(I,P), once(formula_new_stm(I+1, _)),
      \+formula_stm(I, contra(I-1, Alpha, _)),
      \+formula_stm(I, contra(I-1, _, Alpha)),
      \+formula_stm(I, loses(Alpha)),
      Alpha \= now(I),
      (Alpha=not(k(I-1, Beta)) -> \+formula_qtm(I, not(k(I, Beta))) ; true),
      (Alpha=contra(I-1, Beta, Gamma)
          -> \+formula_qtm(I, contra(I, Beta, Gamma)) ; true)],
     belief([stm,0,C,I+1,P,R], Alpha, isi)).

% Inheritance in ITM (II)
rule([belief([itm,0,C,I,P,R], Alpha, _)],
     belief([itm,0,C,I+1,P,max(0,R-1)], Alpha, ii)).

% Semantic retrieval (SR)
rule([belief([stm,0,  _C1,I,_,_], Alpha, _),
      belief([ltm,Beta,C2,I,0,0], Gamma, _),
      csf(Alpha, Beta)],
     belief([qtm,0,C2,I,0,0], Gamma, sr)).

% Modus ponens (MP)
rule([belief([stm,0,C1,I,_,R], Alpha, _),
      belief([stm,0,C2,I,_,R], Alpha -> Beta, _)],
     belief([qtm,0,min(C1,C2),I,0,0], Beta, mp)).

% Extended modus ponens (EMP)
rule([belief([stm,0,C1,I,_,R], Ps -> Q, _), 
      all(conjunct(P, Ps), emp_premise([C1,C2],I,R, P)),
      (C1=C2 ; C1\=C2)],
     belief([qtm,0,min(C1,C2),I,0,0], Q, emp)) :- Ps=and(_,_).

% Negative introspection (NI)
rule([belief([stm,0,_,I,_,_], Form1, _),
      csf(Form2, Form1), Form2 \= k(_,_),
      compound(Form2), Form2 \= and(_,_),
      \+belief([stm,0,_,I,_,_], Form2, _)],
     belief([qtm,0,1,I,0,0], not(k(I, Form2)), ni)).

% Contradiction detection
% same certainty (CD1)
rule([belief([stm,0,C,I,_,R], Alpha, _),
      belief([stm,0,C,I,_,R], not(Alpha), _)],
     belief([qtm,0,C,I,0,0], contra(I, Alpha, not(Alpha)), cd1)).
% different certainties (CD2A, CD2B)
rule([belief([stm,0,0,I,_,R], Alpha, _),
      belief([stm,0,1,I,_,R], not(Alpha), _)],
     belief([qtm,0,1,I,0,0], loses(Alpha), cd2a)).
rule([belief([stm,0,0,I,_,R], not(Alpha), _),
      belief([stm,0,1,I,_,R], Alpha, _)],
     belief([qtm,0,1,I,0,0], loses(not(Alpha)), cd2b)).

% EMP special premise.
% prove one P in Ps
emp_premise(Cs,I,R, P) :-
  query(belief([stm,0,C,I,_,R], P, _)), member(C, Cs).

% Memory bank sets
% in f_formulas(S_QTM(i))
formula_qtm(I, Form) :- query(belief([qtm,0,_,I,_,_], Form, _)).
% in f_formulas(S_STM(i))
formula_stm(I, Form) :- query(belief([stm,0,_,I,_,_], Form, _)).
% in f_formulas(S_new-STM(i))
formula_new_stm(I, Form) :- query(belief([stm,0,_,I,I,_], Form, _)).
% in f_formulas(S_RTM(i))
formula_rtm(I, Form) :- query(belief([stm,0,_,I,_,_], Form, _)).
formula_rtm(I, Form) :- query(belief([itm,0,_,I,_,R], Form, _)), R>0.

% "A simple way to define f_min-STM-pos(i) would be to set it to max(0,i-S+1),
% where S is the intended maximum number of elements in STM."
% (Asker and Malec, 2005 AAAI)
min_stm_pos(I, max(0,I-S+1)) :- S=20.

portray(belief(Label, Formula, RuleTag)) :-
  print(Label), write(' : '),
  print_formula(Formula),
  write(' ('), print(RuleTag), write(')').
