:- [altp].

% Run one step in Julien's prover.
% (You should first load ActiveLogics.pl and arguments.txt)
step :-
  arguments(Rules1,Rules2,Axioms,Obs),
  (recorded(beliefs, Beliefs1, Ref) ->
    (erase(Ref),
     apply_all_rules(Rules1,Beliefs1,Obs,Beliefs2),
     apply_all_rules(Rules2,Beliefs2,Obs,Beliefs3), 
     append(Beliefs2,Beliefs3,Beliefs)) ;
    (apply_all_rules(Rules2,Axioms,Obs,Concls),
     append(Axioms,Concls,Beliefs))),
  !,
  recordz(beliefs, Beliefs),
  forall(member(X,Beliefs),
    ((beliefsyntax(X,Y) -> print(Y) ; print(X)),nl)).

% Convert Julien's syntax of beliefs

beliefsyntax([I,X], belief(I,Y)) :-
  integer(I), beliefsyntax(X,Y).
beliefsyntax([I,X,Rule], belief(I,Y,Rule)) :-
  integer(I), beliefsyntax(X,Y).
beliefsyntax([L,Trig1,C,I,P,R,Form1,Rule],
             belief([L,Trig2,C,I,P,R],Form2,Rule)) :-
  beliefsyntax(Trig1, Trig2),
  beliefsyntax(Form1, Form2).

beliefsyntax(X, X) :- \+ compound(X).
beliefsyntax(any(N), any(N)).
beliefsyntax([and|[X|Xs]], and(Y,Ys)) :-
  (nonvar(Xs) ; nonvar(Ys)),
  beliefsyntax(X,Y), beliefsyntax([and|Xs],Ys).
beliefsyntax([not,X], not(Y)) :- beliefsyntax(X,Y).
beliefsyntax([Name|Args], Bin) :-
  (nonvar(Name) ; nonvar(Bin)),
  member(Name, ['and','->','+','-']),
  functor(Bin, Name, 2),
  beliefsyntax(Args, Bin, 1).
beliefsyntax(pred(Name, Args), Pred) :-
  (nonvar(Name), nonvar(Args), length(Args, Arity) ; nonvar(Pred)),
  functor(Pred, Name, Arity), Arity > 0,
  beliefsyntax(Args, Pred, 1).

beliefsyntax(Args, Term, I) :-
  arg(I, Term, Y), nth1(I, Args, X), beliefsyntax(X,Y),
  succ(I,J), beliefsyntax(Args, Term, J).
beliefsyntax(Args, Term, N) :-
  functor(Term, _, N), length(Args, N),
  arg(N, Term, Y), last(Args, X), beliefsyntax(X,Y).

portray(belief(I, Formula)) :-
  print(I), write(' : '),
  print_formula(Formula).
portray(belief(Label, Formula, RuleTag)) :-
  print(Label), write(' : '),
  print_formula(Formula),
  write(' ('), print(RuleTag), write(')').
