#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include "tetris.h"
#include "draw.h"
#include "textgfx/textgfx.h"

/* Windows */
#define WIN_NEXT 3
#define WIN_PANEL 5

char tetrom_colors[7] = {
	1,  /* I red */
	7,  /* J white */
	5,  /* L magenta */
	4,  /* O blue */
	2,  /* S green */
	3,  /* T yellow/brown */
	6   /* Z cyan */
};

static int twoplayer_mode;

static void drawbl(int bl, int clr, int x, int y)
{
	int c, i;
	setblockcolor(clr%10);
	i = clr<10;
	c = block_chars[i];
	while (bl) {
		setcurs(x, y);
		i = 1;
		do {
			if (!(bl & i))
				movefwd(2);
			else {
				putch(block_chars[0]);
				putch(c);
			}
		} while (bl & 16-(i <<= 1));
		bl >>= 4;
		y++;
	}
}

static void drawstr(const char *str, int n, int x, int y)
{
	char s[8];
	const char *p;
	int m;
	while (p = strchr(str, '\\')) {
		m = p-str;
		memcpy(s, str, m);
		s[m] = '\0';
		str += m+1;
		if (isdigit(p[1])) {
			str++;
			m = p[1]-'0';
		} else
			m = 1;
		do {
			printstr_acs(s, n);
			setcurs(x, ++y);
		} while (--m);
	}
	printstr_acs(str, n);
}

static int board_x(int pl, int col)
{
	int x = 2*col;
	if (pl == 2)
		return x+35;
	if (twoplayer_mode)
		return x+1;
	return x+11;
}

static void drawboard(int pl)
{
	int x = board_x(pl, 0);
	int i;
	definewin(pl, x, 0, 20, 20);
	x--;
	setwin(0);
	setcurs(x, 0);
	setcolorpair(BOARD_FRAME_COLOR);
	for (i = 0; i < 20; i++) {
		putch(VLINE);
		movefwd(20);
		putch(VLINE);
		newln(x);
	}
}

static int draw_vline(int x, int y, int h)
{
	if (is_outside_screen(x+1, 0))
		return 0;
	movefwd(x);
	while (--h) {
		putch(VLINE);
		newln(x);
	}
	putch(VLINE);
	return 1;
}

static void drawboard_frame_2p()
{
	int x = board_x(2, 10)+1;
	int y = 0;
	setcurs(0, y);
	setcolorpair(BOARD_FRAME_COLOR);
	setattr_bold();
	if (draw_vline(x, y, 20)) {
		x = board_x(1, 0)-2;
		setcurs(x, y);
		drawstr("x\\9x\\9x\\x", 0, x, y);
	}
}

static void drawpanel_labels(const char *first, int x)
{
	const char *s = first;
	int i;
	int fg = 3;
	int bg = 4;
	if (!twoplayer_mode) {
		bg = (player1.level % 6)+1;
		if (bg==6)
			fg = 7;
	}
	initcolorpair(PANEL_LABEL_COLOR, bg, fg, 1);
	setcolorpair(PANEL_LABEL_COLOR);
	for (i = 1; i <= 9; i += 4) {
		setcurs(x, i);
		if (s[0]==' ')
			movefwd(1);
		else
			putch(' ');
		printstr(s);
		putch(' ');
		s = s==first ? "Level" : "Lines";
	}
}

static void printstat_1p()
{
	setattr_normal();
	if (!_WHITE_BG)
		setattr_bold();
	setcurs(1, 2);
	printlong(" %06ld ", player1.score % 1000000);
	setcurs(3, 6);
	printint(" %02d ", player1.level);
	setcurs(3, 10);
	printint(" %03d ", player1.lines);
}

static void printstat_2p(Player p)
{
	int x;
	setattr_normal();
	if (!_WHITE_BG)
		setattr_bold();
	x = isplayer2(p) ? 7 : 2;
	setcurs(x, 2);
	printint(" %d ", p->score);
	if (x == 2)
		x--;
	setcurs(x, 6);
	printint(" %02d ", p->level);
	setcurs(x, 10);
	printint(" %02d ", p->lines);
}

static void drawpanel_bordercolor(int clr)
{
	setcolorpair(clr | 16);
}

static void drawpanel_1p()
{
	const char *firstlabel = "Score";
	int clr = (player1.level % 6)+1;
	int i;
	setwin(WIN_PANEL);
	setcurs(0, 0);
	setblockcolor(clr);
	printstr_acs("lqNu", 8);
	for (i = 0; i <= 8; i += 4)
		drawstr("\\xhNx\\3tqNu", 8, 0, i);
	setcurs(0, 13);
	printstr_acs("xhNx", 8);
	drawpanel_labels(firstlabel, 1);
	setcurs(2, 13);
	printstr(" Next ");
	printstat_1p();
	drawpanel_bordercolor(clr);
	drawstr("\\x Nx\\3mqNu", 8, 0, 13);
	drawstr("\\x\\x", 0, 9, 17);
	setwin(0);
	setcurs(0, 0);
	drawpanel_bordercolor(clr);
	draw_vline(board_x(1, 10)+1, 0, 20);
}

static void drawpanel_2p()
{
	int i;
	setwin(WIN_PANEL);
	setcurs(0, 0);
	setblockcolor(4);
	printstr_acs("tqNu", 10);
	for (i = 0; i <= 8; i += 4)
		drawstr("\\xhNx\\3tqNu", 10, 0, i);
	drawpanel_labels("Wins", 3);
	printstat_2p(&player1);
	printstat_2p(&player2);
	drawpanel_bordercolor(4);
	drawstr("\\x Nx\\6x Nx", 10, 0, 12);
	refreshwin();
}

void drawgamescreen_1p()
{
	definewin(WIN_NEXT, 1, 15, 8, 2);
	definewin(WIN_PANEL, 0, 0, 10, 20);

	twoplayer_mode = 0;
	drawboard(1);
	refreshwin();
	redrawboard(&player1, 19);
	drawpanel_1p();
	setwin(WIN_PANEL);
	refreshwin();
}

void drawgamescreen_2p()
{
	twoplayer_mode = 1;
	drawboard(1);
	drawboard(2);
	drawboard_frame_2p();
	refreshwin();
	redrawboard(&player1, 19);
	redrawboard(&player2, 19);
	drawpanel_2p();
}

void drawnext(Player p, Tetr next)
{
	int bl;
	int x;
	if (twoplayer_mode && !_HEIGHT_24L && term_width < 76)
		return;
	setwin(WIN_NEXT+isplayer2(p));
	clearbox(0, 0, 8, 2);
	if (next && (bl = next->blocks)) {
		x = (bl!=TETR_I && bl!=TETR_O);
		while (!(bl & 15))
			bl >>= 4;
		setcurs(x, 0);
		drawbl(bl, next->color+10, x, 0);
	}
	refreshwin();
	setcurs_end();
}

static void drawboard_bg(Player p, int col)
{
	int lefthand = p->rotation & ROT_LEFTHAND;
	putch( (lefthand && col%2) ? bgdot : ' ');
	putch(!(lefthand || col%2) ? bgdot : ' ');
}

static void setboardwin(Player p)
{
	setwin(1+isplayer2(p));
}

void drawblocks(Player p, int bl, int x, int y, Drawmode mode)
{
	int i, j;
	x *= 2;
	while (y < 0) {
		bl >>= 4;
		y++;
	}
	setboardwin(p);
	setcurs(x, y);
	if (mode != CLEAR_BLOCKS) {
		i = p->piece.color;
		if (mode == DRAW_PIECE)
			i += 10;
		drawbl(bl, i, x, y);
	} else {
		setblockcolor(0);
		while (1) {
			i = 1;
			j = x/2;
			do {
				if (bl & i)
					drawboard_bg(p, j);
				else
					movefwd(2);
				j++;
			} while (bl & 16-(i <<= 1));
			if (!(bl >>= 4))
				break;
			y++;
			setcurs(x, y);
		}
	}
}

void drawrow(Player p, int y)
{
	uint_least32_t bl = p->board[y];
	int clr = -1;
	int i = 0;
	setboardwin(p);
	setcurs(0, y);
	do {
		if (clr != (bl & 7)) {
			clr = bl & 7;
			setblockcolor(clr);
		}
		if (clr) {
			putch(block_chars[0]);
			putch(block_chars[1]);
		} else
			drawboard_bg(p, i);
		bl >>= 3;
	} while (++i < 10);
}

void redrawboard(Player plr, int y)
{
	while (y >= 0) {
		drawrow(plr, y);
		y--;
	}
	if (plr->piece.blocks)
		drawpiece(plr, plr->piece.blocks, plr->piece.x, plr->piece.y);
	refreshwin();
}

void upd_screen(Player p)
{
	setboardwin(p);
	refreshwin();
	setcurs_end();
}

void upd_stat(Player p, int levelup)
{
	setwin(WIN_PANEL);
	setcurs(1, 2);
	if (twoplayer_mode)
		printstat_2p(p);
	else {
		if (levelup)
			drawpanel_1p();
		else
			printstat_1p();
	}
	setwin(WIN_PANEL);
	refreshwin();
}

static void drawredmeter(int x, int n)
{
	int i = 7;
	int c = ' ';
	while (i) {
		if (i <= n) {
			c = UPARROW;
			setblockcolor(1);
			n = 0;
		}
		newln(x);
		putch(c);
		putch(c);
		i--;
	}
	setattr_normal();
}

void upd_garbagemeter(Player p, int n)
{
	char digits[3] = "  ";
	int x = isplayer2(p) ? 9  : 1;
	if (n) {
		digits[0] = '0'+ n/10;
		digits[1] = '0'+ n;
	}
	setwin(WIN_PANEL);
	setcurs(0, 12);
	drawredmeter(x, n);
	if (n > 6)
		n = 6;
	setcurs(x, 19-n);
	printstr(digits);
	refreshwin();
}

void show_winner(Player p)
{
	int i = 1;
	while (1) {
		setwin(i);
		setcurs(4, 2);
		setattr_normal();
		setcolorpair(MAGENTA_FG);
		drawbox(4, 2, 12, 3, (char *)0);
		setcurs(6, 3);
		if (!p)
			printstr("  DRAW  ");
		else {
			printstr("YOU ");
			printstr(p==&game.player[i==2] ? "WIN!" : "LOSE");
		}
		refreshwin();
		if (i == 2)
			break;
		i = 2;
	}
	setcurs_end();
}
