/* Text mode output using ANSI/VT100 escape sequences directly */

#include <stdio.h>
#include <string.h>
#include "textgfx.h"

unsigned textgfx_flags = 0;

static char curs_x;
static char curs_y;
static char linestart[25];
static int  inited;

void set_input_mode();
void restore_input_mode();

void textgfx_init()
{
	set_input_mode();
	printf("\033[?25l");	/* hide cursor */
	inited = 1;
}

void textgfx_end()
{
	if (!inited)
		return;
	setattr_normal();
	setcurs_end();
	putch(' ');	/* switch back from acs */
	restore_input_mode();
	printf("\033[?25h\033[m\n");
	inited = 0;
}

static void set_curs(int x, int y)
{
	if (x != curs_x) {
		if (x > curs_x)
			printf("\033[%dC", x-curs_x);
		else
			printf("\033[%dD", curs_x-x);
		curs_x = x;
	}
	if (y != curs_y) {
		if (y < curs_y)
			printf("\033[%dA", curs_y-y);
		else
			printf("\033[%dB", y-curs_y);
		curs_y = y;
	}
}

void setcurs(int x, int y)
{
	x += win_begin_x;
	y += win_begin_y;
	set_curs(x, y);
}

void movefwd(int n)
{
	printf("\033[%dC", n);
	curs_x += n;
}

void newln(int x)
{
	putchar('\n');
	x += win_begin_x;
	if (x > 0)
		printf("\033[%dC", x);
	curs_x = x;
	curs_y++;
}

static int getend_y()
{
	int y = 24;
	while (y > 0 && !linestart[y])
		y--;
	if (y >= term_height)
		y = term_height-1;
	return y;
}

void setcurs_end()
{
	set_curs(0, getend_y());
	fflush(stdout);
}

int is_outside_screen(int x, int y)
{
	return x + win_begin_x >= term_width ||
	       y + win_begin_y >= term_height;
}

void get_xy(int *x, int *y)
{
	*x = curs_x - win_begin_x;
	*y = curs_y - win_begin_y;
}

void refreshwin()
{
	fflush(stdout);
}

void clearscreen()
{
	int y = getend_y();
	for (; y>=0; y--) {
		set_curs(0, y);
		cleartoeol();
	}
}

void cleartoeol()
{
	int y = curs_y;
	printf("\033[K");
	if (y >= 0 && y < 25 && curs_x < linestart[y])
		linestart[y] = 0;
}

static void set_ansi_color(int bg, int fg, int bold)
{
	if (bg >= 0)
		printf("\033[4%cm", bg+'0');
	else
		printf("\033[m");
	printf("\033[%c;3%cm", bold+'0', fg+'0');
}

void set_color_pair(int pair, int bg, int fg, int bold)
{
	if (pair == 0x17)
		printf("\033[m\033[1m");
	else
		set_ansi_color(bg, fg, bold);
}

void setattr_normal()
{
	printf("\033[m");
}

void setattr_standout()
{
	printf("\033[7m");
}

void setattr_bold()
{
	printf("\033[1m");
}

void setattr_underline()
{
	printf("\033[4m");
}

static void setlinestart()
{
	int y = curs_y;
	if (y >= 0 && y < 25 && (!linestart[y] || curs_x < linestart[y]))
		linestart[y] = curs_x;
}

void put_ch(int ch)
{
	static int acs;
	if (ch < 0)
		ch = (unsigned char) ch;
	if (ch & 0x100) {
		if (_ASCII) {
			putch_ascii(ch);
			return;
		}
		if (ch == (' '|0x100) || ch == TEXTURE2 && !_LINUX_TERM)
			ch = ' ';
		else if (!acs) {
			printf("\033(0");
			acs = 1;
		}
		ch &= ~0x100;
	} else if (acs) {
		printf("\033(B");
		acs = 0;
	}
	putchar(ch);
	if (ch=='\b')
		curs_x--;
	else {
		curs_x++;
		setlinestart();
	}
}

int printstr(const char *str)
{
	int len = strlen(str);
	int i;
	if (!len)
		return 0;
	if (curs_x+len >= term_width) {
		len = term_width-curs_x-1;
		if (len < 1)
			return 0;
	}
	putch(str[0]);
	for (i = 1; i < len; i++) {
		putchar(str[i]);
		curs_x++;
	}
	return 1;
}

void printint(const char *fmt, int d)
{
	putch(' ');
	putch('\b');
	curs_x += printf(fmt, d);
}

void printlong(const char *fmt, long d)
{
	putch(' ');
	putch('\b');
	curs_x += printf(fmt, d);
}
