#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include "textgfx.h"

char term_width  = 80;
char term_height = 20;

static int gettermsize_unix()
{
#if defined(TIOCGSIZE)
	struct ttysize tty;
	if (!ioctl(STDIN_FILENO, TIOCGSIZE, &tty)) {
		settermwidth (tty.ts_cols);
		settermheight(tty.ts_lines);
		return 1;
	}
#elif defined(TIOCGWINSZ)
	struct winsize win;
	if (!ioctl(STDIN_FILENO, TIOCGWINSZ, &win)) {
		settermwidth (win.ws_col);
		settermheight(win.ws_row);
		return 1;
	}
#endif
	return 0;
}

void gettermsize()
{
	char *s;
	int w = 80;
	int h = 25;
	if (gettermsize_unix())
		return;
	if (s = getenv("COLUMNS"))
		w = atoi(s);
	if (s = getenv("LINES"))
		h = atoi(s);
	settermwidth(w);
	settermheight(h);
}

void settermwidth(int w)
{
	if (w < 32)
		w = 32;
	else if (w > 127)
		w = 127;
	term_width = w;
}

void settermheight(int h)
{
	if (h < 20)
		h = 20;
	else if (h > 25)
		h = 25;
	term_height = h;
	if (term_height < 24)
		textgfx_flags &= ~HEIGHT_24L;
	else
		textgfx_flags |= HEIGHT_24L;
}

void gettermenv()
{
	char *s = getenv("TERM");
	if (s) {
		if (!strncmp(s, "xterm", 5))
			textgfx_flags |= XTERM;
		else if (!strcmp(s, "linux"))
			textgfx_flags |= LINUX_TERM;
		else if (!strcmp(s, "cygwin"))
			textgfx_flags |= CYGWIN;
	}
	s = getenv("COLORTERM");
	if (s && !strcmp(s, "gnome-terminal"))
		textgfx_flags |= GNOME_TERM;
}
