/* VITETRIS TEXTGFX LIBRARY
   Text display output and control */

/* +-----------------+
 * |  textgfx_flags  |
 * +-----------------+ */
#define ASCII      1
#define WHITE_BG   2
#define MONOCHROME 4
#define HEIGHT_24L 8

/* block styles */
#define BLACK_BRACKETS 16
#define TT_BLOCKS      32
#define TT_BLOCKS_BG   64

#define XTERM      0x80
#define LINUX_TERM 0x100
#define GNOME_TERM 0x200
#define CYGWIN     0x400

extern unsigned textgfx_flags;

#define _ASCII	      (textgfx_flags & ASCII)
#define _WHITE_BG     (textgfx_flags & WHITE_BG)
#define _MONOCHROME   (textgfx_flags & MONOCHROME)
#define _HEIGHT_24L   (textgfx_flags & HEIGHT_24L)
#define _TT_BLOCKS    (textgfx_flags & TT_BLOCKS)
#define _TT_BLOCKS_BG (textgfx_flags & TT_BLOCKS_BG)
#define TT_MONO       (TT_BLOCKS | MONOCHROME)
#define _XTERM        (textgfx_flags & XTERM)
#define _LINUX_TERM   (textgfx_flags & LINUX_TERM)

extern char term_width;
extern char term_height;

extern char win_begin_x;
extern char win_begin_y;

extern short block_chars[2];
extern short bgdot;
extern short default_bgdot;

void gettermsize();
void settermwidth(int cols);
void settermheight(int lines);
void gettermenv();

void textgfx_init();
void textgfx_end();

void definewin(int win, int begin_x, int begin_y, int cols, int lines);
void setwin(int win);	/* set current window */
void refreshwin();	/* refresh current window */

/* move cursor - x and y are relative to the current window */
void setcurs(int x, int y);
void movefwd(int n);
void newln(int x);
void setcurs_end();
int is_outside_screen(int x, int y);
void get_xy(int *x, int *y);

void clearscreen();
void cleartoeol();

/* Color pairs.
 * 1-7 are solid red, green, yellow, blue, magenta, cyan, white (black).
 * 0x11-0x17 are foreground colours. */
#define MAGENTA_FG        8
#define WHITE_ON_BLUE     9
#define BOARD_BG_COLOR    10
#define BOARD_FRAME_COLOR 11
#define PANEL_LABEL_COLOR 12
#define RED_FG            13
#define YELLOW_ON_GREEN   14
#define YELLOW_ON_BLUE    15

/* change color pair */
void initcolorpair(int pair, int bg, int fg, int bold);
/* use color pair */
void setcolorpair(int pair);
/* inner function */
void set_color_pair(int pair, int bg, int fg, int bold);

/* Set color pair and block chars based on block style.
 * color should be 1-7, or 0 for board bg. */
void setblockcolor(int color);

void reset_block_chars();

void setattr_normal();
void setattr_standout();
void setattr_bold();
void setattr_underline();

/* Line drawing characters - used with putch */

#define UPLEFT	   ('l' | 0x100)
#define LOWLEFT    ('m' | 0x100)
#define UPRIGHT	   ('k' | 0x100)
#define LOWRIGHT   ('j' | 0x100)
#define CROSSLINES ('n' | 0x100)
#define HLINE	   ('q' | 0x100)
#define VLINE	   ('x' | 0x100)
#define LEFT_T	   ('t' | 0x100)
#define RIGHT_T	   ('u' | 0x100)
#define BOTTOM_T   ('v' | 0x100)
#define TOP_T	   ('w' | 0x100)
#define TEXTURE1   ('a' | 0x100)	/* ACS_CKBOARD */
#define TEXTURE2   ('h' | 0x100)	/* ACS_BOARD or blank */
#define BULLET	   ('~' | 0x100)
#define UPARROW	   ('-' | 0x100)

/* convert to codepage 437 */
int ibmgfx(int ch);

/* putch conflicts with DOS conio */
void put_ch(int ch);
#ifndef conio_h
#define putch(ch) put_ch(ch)
#endif
int printstr(const char *str);
void printint(const char *fmt, int d);
void printlong(const char *fmt, long d);

/* takes a line drawing character and prints ascii */
void putch_ascii(int ch);

int putnchars(int ch, int n);

/* prints lowercase letters in str using alternate character set,
 * and repeats a character followed by 'N' n times */
void printstr_acs(const char *str, int n);

int isprintable(int c);

void drawbox(int x, int y, int w, int h, const char *title);
void clearbox(int x, int y, int w, int h);
